from datetime import datetime
from colorama import init, Fore, Back
import os
from hashlib import sha256
from .tipagem import iniciarTipagem

init()


parametrosBloqueados = []
gerarLogs = False
estadoConsoleLog = False
caracteresSeparadores = [' ']
palavrasBloqueadas = []
comandosSQLInjection = ['WHERE','FROM','SELECT','CREATE',';','=',"'",'"','`','AND','OR','ALTER','TABLE','GROUP BY','NOT','!','--']


def src_RetornoHora():
    hora_atual = datetime.now()
    hora_atual = hora_atual.time()
    hora_atual = str(hora_atual)

    return hora_atual[0:8]


def src_RetornoData(formato):
    data_atual = datetime.now()
    data_atual = data_atual.date()
    data_atual = data_atual.strftime(formato)

    return data_atual


def stringVazia(*strings):
    for item in strings:
        if (len(item)) <=0:
            return False
        else:
            contador = 0
            for letra in range(len(item)):
                if item[letra] == ' ':
                    contador += 1
            
            if contador >= len(item):
                return True
            else:
                return False


# Filtrar Parametros URL Banidos:

def bloquearParametro(*valoresParametro):
    for item in valoresParametro:
        item = item.upper()
        parametrosBloqueados.append(item)


def parametroBanido(*valoresParametros):
    for item in valoresParametros:
        item = item.upper()
        if item in parametrosBloqueados:
            return True
    else:
        return False
    

# Validar CPF:


# Criar Log's De Testes:
    
def configLog(criarLogs:bool=False,exibirLogsEmConsole:bool=False):
    global gerarLogs
    global estadoConsoleLog
    gerarLogs = criarLogs
    estadoConsoleLog = exibirLogsEmConsole


def statusLog():
    global gerarLogs
    return gerarLogs

    
def gerarLog(tipo:str,conteudo:str):
    if gerarLogs == True:
        data_atual = src_RetornoData("%d-%m-%Y")
        hora_atual = src_RetornoHora()

        tipo = tipo.upper()

        conteudo = f'{tipo}: {conteudo} || DATA: {data_atual} || HORA: {hora_atual}\n'
        with open('log.txt', 'a') as arquivo:
            arquivo.write(conteudo)

        if estadoConsoleLog == True:
            os.system("cls")
            print(Fore.RED + conteudo)

        return True
    else:
        print(Fore.RED + "ERRO: Gerar Log está desativado! Para ativar use a função 'configLog()'")


def quebrarFrase(frase):
    palavras = []
    controle = 0
    tamanhoFrase = len(frase)
    for i in range(tamanhoFrase):
        item = frase[i]
        if ((item in caracteresSeparadores) or (i == (tamanhoFrase-1))):
            if (i == (tamanhoFrase-1)):
                i +=1
            palavra = frase[controle:i]
            palavras.append(palavra)
            controle = i+1

    return palavras


# Anti SQL Injection

def antiSQLInjection(*parametros):
    for parametro in parametros:
        parametro = parametro.upper()
        isolados = quebrarFrase(parametro)
        for i in range(len(isolados)):
            if (isolados[i] in comandosSQLInjection) or (isolados[i] in palavrasBloqueadas):
                return [False, 'Parametro potencialmente perigoso']
        else:
            for letra in parametro:
                if (letra in comandosSQLInjection) or (letra in palavrasBloqueadas):
                    return [False, 'Parametro potencialmente perigoso']
    else:
        return [True, 'Parametro válido']


def bloquearPalavraSQLInjection(*palavra):
    for item in palavra:
        item = item.upper()
        palavrasBloqueadas.append(item)
    return True


# Criptografia

def criptoToSHA256(arg:str):
    hash = sha256(arg.encode())
    resultado = hash.hexdigest()
    return resultado


from .geral import gerarLog,statusLog,configLog
from colorama import init, Fore, Back
import linecache

init()


variaveisTipadas = []
tiposPossiveis = ['bool','int','str','float']
constantesCriadas = []


def verificarTipo(valor):
    tipo = type(valor)
    if tipo == str:
        return 'str'
    elif tipo == int:
        return 'int'
    elif tipo == bool:
        return 'bool'
    elif tipo == float:
        return 'float'


def declararVariavel(nomeVariavel:str,tipoVariavel:str,valorInicial=None):
    tipoVariavel = tipoVariavel.lower()

    if tipoVariavel in tiposPossiveis:
        if valorInicial == None:
            if tipoVariavel == 'bool':
                valorInicial = False
            elif tipoVariavel == 'str':
                valorInicial = ''
            elif tipoVariavel == 'float':
                valorInicial = 0.0
            elif tipoVariavel == 'int':
                valorInicial = 0

        tipo = verificarTipo(valorInicial)
        
        if tipo == tipoVariavel:
            newVariavel = {'nome':nomeVariavel,'tipo':tipoVariavel,'valor':valorInicial}    
            variaveisTipadas.append(newVariavel)

            return valorInicial
        else:
            erro = f'ERRO: O tipo inicial é "{tipoVariavel}", enquanto o valor inicial é do tipo "{tipo}"'
            print(Fore.YELLOW + erro)
            logAtivado = statusLog()
            if logAtivado == True:
                configLog(criarLogs=True,exibirLogsEmConsole=False)
                gerarLog('CRITICAL',erro)
            return None
    else:
        erro = f'ERRO: Não é possivel usar o tipo "{tipoVariavel}"'
        print(Fore.RED + erro)
        logAtivado == statusLog()
        if logAtivado == True:
            configLog(criarLogs=True,exibirLogsEmConsole=False)
            gerarLog('CRITICAL',erro)
        return None


def atribuirValor(nomeVariavel:str, valor):
    for i in range(len(variaveisTipadas)):
        if nomeVariavel == variaveisTipadas[i]['nome']:
            tipoDeclarado = variaveisTipadas[i]['tipo']
            tipoPassado = verificarTipo(valor)
            if tipoDeclarado == tipoPassado:
                variaveisTipadas[i]['valor'] = valor
                return valor
            else:
                erro = f"ERRO: Você está tentando atribuir um valor do tipo '{tipoPassado}' numa variável do tipo '{tipoDeclarado}'"
                print(Fore.YELLOW + erro)
                logAtivado = statusLog()
                if logAtivado == True:
                    configLog(criarLogs=True,exibirLogsEmConsole=False)
                    gerarLog('CRITICAL',erro)
                return None


def declararCostante(nome:str,valor):
    if (len(constantesCriadas)) <=0:
        novaConstante = {'nome':nome,'valor':valor,'qtd':1}
        constantesCriadas.append(novaConstante)
        return valor
    else:
        for i in range(len(constantesCriadas)):
            if nome == constantesCriadas[i]['nome']:
                erro = f"Constante {nome} já existe e seu valor não pode ser alterado"
                print(Fore.YELLOW + erro)
                logAtivado = statusLog()
                if logAtivado == True:
                    configLog(criarLogs=True,exibirLogsEmConsole=False)
                    gerarLog('CRITICAL',erro)
                return None
        else:
            novaConstante = {'nome':nome,'valor':valor}
            constantesCriadas.append(novaConstante)
            return valor


def iniciarTipagem(caminho):

    with open(caminho, 'r') as arquivo:
        numeroDeLinhas = len(arquivo.readlines())
    
    for i in range(numeroDeLinhas):
        conteudoLinha = linecache.getline(caminho,i)
        quantidadeConst = len(constantesCriadas)
        for item in range(quantidadeConst):
            nomeConstante = constantesCriadas[item]['nome']
            vezesUsada = constantesCriadas[item]['qtd']

            possibilidade1 = f'{nomeConstante}='
            possibilidade2 = f'{nomeConstante} ='
            possibilidade3 = f'{nomeConstante}  ='

            if (possibilidade1 in conteudoLinha) or (possibilidade2 in conteudoLinha) or (possibilidade3 in conteudoLinha):
                constantesCriadas[item]['qtd'] = vezesUsada+1
            
            vezesUsada = constantesCriadas[item]['qtd']

            if (vezesUsada >=2):
                erro = f'ERRO: A Constante {nomeConstante} está sendo atribuida mais de uma vez'
                print(Fore.RED + erro)
                logAtivado = statusLog()
                if logAtivado == True:
                    configLog(criarLogs=True,exibirLogsEmConsole=False)
                    gerarLog('CRITICAL',erro)
                exit()


import mysql.connector
from datetime import datetime 
from .geral import src_RetornoData,src_RetornoHora,criptoToSHA256

host_name = ""
user_name = ""
password_name = ""
database_name = ""
conexao = ""
cursor = ""


def sql_conectar(host_i:str,user_i:str,password_i:str,database_i:str):
    global host_name
    global user_name
    global password_name
    global database_name
    global cursor
    global conexao

    host_name = host_i
    user_name = user_i
    password_name = password_i
    database_name = database_i

    conexao = mysql.connector.connect(
    host= host_name,    
    user= user_name,
    password= password_name,
    database= database_name
    )
    cursor = conexao.cursor()



def src_verificarUsoEmail(tabela:str, email:str):
    conexao.commit()

    comando = f'SELECT * FROM {tabela} WHERE email = "{email}"'
    cursor.execute(comando)
    resultado = cursor.fetchall()

    if ((len(resultado)) != 0):
        return True
    else:
        return False


def src_RetornoIdUsuario(tabela:str, campoId:str, campoEmail:str, valorEmail:str):
    conexao.commit()

    comando = f'SELECT {campoId} FROM {tabela} WHERE {campoEmail} = "{valorEmail}"'
    cursor.execute(comando)
    id_usuario = cursor.fetchall()
    id_usuario = id_usuario[0][0]

    return int(id_usuario)

  
def src_verificarValidadeEmail(email:str):
  if '@' in email and '.' in email:
    return True
  else:
    return False
  

def sql_criarConta(tabela:str, campoNome:str, campoEmail:str, campoSenha:str, valorNome:str, valorEmail:str, valorSenha:str):
    try: 
        emailUtilizado = src_verificarUsoEmail(tabela,valorEmail)

        if (emailUtilizado == True):
            return [False, 'Email Utilizado']
        
        else:
            emailValido = src_verificarValidadeEmail(valorEmail)

            if (emailValido == True):
                valorSenha = criptoToSHA256(valorSenha)

                comando = f'INSERT INTO {tabela} ({campoNome},{campoEmail},{campoSenha}) VALUES("{valorNome}","{valorEmail}","{valorSenha}")'
                cursor.execute(comando)
                conexao.commit()

                return True
            else:
                return [False, 'Insira um email válido']
            
    except:
        return [False, 'Erro ao tentar executar a função']


def sql_logarConta(tabela:str, campoEmail:str, campoSenha:str, valorEmail:str, valorSenha:str):
    try:
        conexao.commit()

        valorSenha = criptoToSHA256(valorSenha)

        comando = f'SELECT * FROM {tabela} WHERE {campoEmail}="{valorEmail}" AND {campoSenha}="{valorSenha}"'
        cursor.execute(comando)
        resultado = cursor.fetchall()

        if ((len(resultado)) !=0):
            return True
        else:
            return [False, 'Conta não encontrada']
    except:
        return [False, 'Erro ao tentar executar a função']
    

def sql_alterarConta(tabela:str,campo:str,valor,condicao:str):
    try:
        conexao.commit()

        comando = f'UPDATE {tabela} SET {campo}={valor} WHERE {condicao}'
        cursor.execute(comando)
        conexao.commit()

        return True
    except:
        return [False, 'Erro ao tentar executar a função']
    

def sql_excluirConta(tabela:str,condicao:str):
    try:
        conexao.commit()
    
        comando = f'DELETE FROM {tabela} WHERE {condicao}'
        cursor.execute(comando)
        conexao.commit()

        return True
    except:
        return [False, 'Erro ao tentar executar a função']
    

def sql_registrarLog(tabelaLOG:str, tabelaUsuario:str, campoIdLog:str, campoMotivoLog, campoDataLog:str, campoHoraLog:str, campoIdUsuario:str, campoEmail:str, valorEmail:str, motivoLog:str):
    try:
        conexao.commit()

        data = src_RetornoData("%Y-%m-%d")
        hora = src_RetornoHora()
        id = src_RetornoIdUsuario(tabelaUsuario,campoIdUsuario,campoEmail,valorEmail)

        comando = f'INSERT INTO {tabelaLOG} ({campoIdLog},{campoMotivoLog},{campoDataLog},{campoHoraLog}) VALUES("{id}","{motivoLog}","{data}","{hora}")'
        cursor.execute(comando)
        conexao.commit()

        return True
    except:
        return [False, 'Erro ao executar a função']